require_dependency "binect_importer/application_controller"
require_dependency "../../lib/binect_importer/onedrive_importer"

import_config   = Rails.application.config.import_config[:onedrive]
ROOT_PATH       = import_config[:ROOT_PATH]
REDIRECT_URL    = import_config[:REDIRECT_URL]
CLIENT_ID       = import_config[:CLIENT_ID]
CLIENT_SECRET   = import_config[:CLIENT_SECRET]

module BinectImporter
        class OneDriveRegistrationsController < ApplicationController
                def start_registration
                        authorization_url = "https://login.microsoftonline.com/common/oauth2/v2.0/authorize?client_id=#{CLIENT_ID}&scope=files.readwrite offline_access&response_type=code&redirect_uri=#{REDIRECT_URL}"
                        redirect_to authorization_url
                end

                def finish_registration
                        code = params[:code]
                        error = params[:error]

                        if error.present?
                                flash[:error] = "#{error}"
                                redirect_to ROOT_PATH
                                return
                        end

                        # Request the access and refresh tokens.
                        params = {
                                "client_id"     => CLIENT_ID,
                                "redirect_uri"  => REDIRECT_URL,
                                "client_secret" => CLIENT_SECRET,
                                "code"          => code,
                                "grant_type"    => OneDriveImporter::GRANT_TYPE_AUTHORIZATION
                        }

                        uri = URI('https://login.microsoftonline.com/common/oauth2/v2.0/token')
                        result = Net::HTTP.post_form(uri, params)

                        response = JSON.parse(result.body)
                        access_token    = response[OneDriveImporter::ACCESS_TOKEN]
                        refresh_token   = response[OneDriveImporter::REFRESH_TOKEN]

                        # Check whethere this user is already authorized
                        importer_data_index = current_user.authenticated_importers.index { |importer_hash| importer_hash[:type] == OneDriveImporter::IMPORTER_TYPE }
                        importer_data = {
                                class_name: "BinectImporter::OneDriveImporter",
                                type: OneDriveImporter::IMPORTER_TYPE,
                                OneDriveImporter::ACCESS_TOKEN => access_token,
                                OneDriveImporter::REFRESH_TOKEN => refresh_token,
                                OneDriveImporter::CLIENT_ID     => CLIENT_ID,
                                OneDriveImporter::REDIRECT_URI  => REDIRECT_URL,
                                OneDriveImporter::CLIENT_SECRET => CLIENT_SECRET
                        }

                        if importer_data_index.nil?
                                # Add new importer_data
                                Rails.logger.info "Saving new OneDrive access token"
                                current_user.authenticated_importers.push(importer_data)
                        else
                                # Replace importer data
                                Rails.logger.info "Updating OneDrive new access token"
                                current_user.authenticated_importers[importer_data_index] = importer_data
                        end
                        unless current_user.save
                                Rails.logger.error  "Unable to persist change: #{current_user.errors}"
                                flash[:error] = "Unable to save data. Please try again"
                        end

                        Rails.logger.info "OneDrive registration succeeded for user: #{current_user}."
                        redirect_to ROOT_PATH
                end

                def unlink
                        # Just delete the importer. The optional sign out request is only useful for deleting browser cookies.
                        importer_data_index = current_user.authenticated_importers.index { |importer_hash| importer_hash[:type] == OneDriveImporter::IMPORTER_TYPE }
                        current_user.authenticated_importers.delete_at(importer_data_index)
                        current_user.save

                        redirect_to ROOT_PATH
                end
        end
end
