require "dropbox_api"

module BinectImporter
    class DropboxImporter
        IMPORTER_TYPE = "dropbox"

        def initialize(user)
            @user = user
        end

        # count files
        def count
            client = self.class.get_dropbox_client(@user)

            # Load all files in root directory:
            result = client.list_folder ""
            fail "Unable to read data from DrobBox" if result.nil?
            result.entries.delete_if { |f| f.to_hash[".tag"] != "file" }.count
        end

        def import
            client = self.class.get_dropbox_client(@user)

            # Load all files in root directory:
            result = client.list_folder ""
            fail "Unable to read data from DrobBox" if result.nil?
            result.entries.each do |f|
                meta_data = f.to_hash
                # Only handle pdf files
                next if meta_data[".tag"] != "file"
                next if File.extname(f.name).downcase != ".pdf"

                file_path = f.path_display

                Rails.logger.info "Import PDF: Load file with path: #{file_path}"
                file_contents = ""
                file_metadata = client.download(f.id) do |chunk|
                    file_contents << chunk
                end
                Rails.logger.info "Import PDF: ready"
                # Call external block that will handle the file.
                is_success = yield( file_contents, File.basename( file_path ), MIME::Types.type_for(File.extname(f.name)).first.content_type, file_metadata.size )

                if is_success
                    # Remove file
                    Rails.logger.info "Remove file: #{file_path}"
                    client.delete(f.id)
                end
            end
        end

        def export(file_data, file_path, mime_type)
            Rails.logger.info "Exporting file '#{file_path}'.."
            client = self.class.get_dropbox_client(@user)
            unless client.nil?
                Rails.logger.info "..put file.."
                response = client.upload(file_path, file_data, :autorename => true)
                Rails.logger.info "..with response '#{response}'"

                return response.is_a?(DropboxApi::Metadata::File)
            else
                Rails.logger.warn "No Dropbox client found for user: #{@user.id}"
            end

            return false
        end

        def self.get_dropbox_client(user)
            Rails.logger.info "get_dropbox_client"
            return nil if user.nil?
            Rails.logger.info "get data.."
            dropbox_data = user.authenticated_importers.select { |importer_hash| importer_hash[:type] == DropboxImporter::IMPORTER_TYPE }
            Rails.logger.info "found data: #{dropbox_data}"
            return nil if dropbox_data.nil? || dropbox_data.first.nil? || dropbox_data.first[:access_token].nil?

            Rails.logger.info "get access token"
            access_token = dropbox_data.first[:access_token]
            Rails.logger.info "it is: #{access_token}"

            ::DropboxApi::Client.new(access_token)
        end
    end
end
